<?php
/*
Plugin Name: Hikari Email & URL Obfuscator
Plugin URI: http://Hikari.ws/email-url-obfuscator/
Description: This plugin obfuscates email and normal links, to hide them from spam harvesters and Search Endigne crawlers. It  uses ROT13 or cc8b to encode data while PHP is building the page, then uses JavaScript to decode it and show it to the user. If JavaScript is not available, it uses CSS obfuscate to hide them. 
Version: 0.04.05
Author: Hikari
Author URI: http://Hikari.ws
*/

/**!
* Thanks to http://www.debuggeddesigns.com/open-source-projects/mailto and
* http://techblog.tilllate.com/2008/07/20/ten-methods-to-obfuscate-e-mail-addresses-compared/
* for their original work.
*
* I, Hikari, from http://Hikari.WS , and the original author of the Wordpress plugin named
* Hikari Email & URL Obfuscator, please keep this license terms and credit me if you redistribute the plugin
*
* Hikari Email & URL Obfuscator,  is licensed under the Creative Commons Attribution-Noncommercial-Share Alike 3.0
* To view a copy of this license, please visit http://creativecommons.org/licenses/by-nc-sa/3.0/
*
*   This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
/*****************************************************************************
*  Copyright Hikari (http://Hikari.WS), 2010
* If you want to redistribute this script, please leave a link to
* http://hikari.WS/
*
* Parts of this code are provided or based on ideas and/or code written by others
* Translations to different languages are provided by users of this script
* IMPORTANT CONTRIBUTIONS TO THIS SCRIPT (listed in alphabetical order):
*
** http://techblog.tilllate.com/2008/07/20/ten-methods-to-obfuscate-e-mail-addresses-compared/
** http://www.debuggeddesigns.com/open-source-projects/mailto
*
* Please send a message to the address specified on the page of the script, for credits
*
* Other contributors' (nick)names may be provided in the header of (or inside) the functions
* SPECIAL THANKS to all contributors and translators of this script !
*****************************************************************************/

//mail regex:
// [a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+(?:[A-Z]{2}|com|org|net|gov|mil|biz|info|mobi|name|aero|jobs|museum)\b
// \b[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}\b

define('HKMUOB_FOLDER', '/hikari-email-url-obfuscator'); //The name of the plugin folder, change this if you rename it


class HkMuob{

	private $links;
	private $ids;
	private $plugin_url;
	private $specialParameters;
	private $jsObfuscation;
	private $cssObfuscation;
	public $true_page;


	/*
		Class constructor, it is run when the plugin is loaded (object is instantiated on the bottom of the code)
		It defines the plugin URL path (so that js and css can be loaded by the browser) and adds our filter to proper hooks.
	*/
	function __construct(){
		$this->plugin_url = plugins_url(HKMUOB_FOLDER);
	
		$this->setFilters();
	}
	

	// add our filter to proper hooks
	// priority must be > 1000 to not conflict with CodeColorer http://kpumuk.info/projects/wordpress-plugins/codecolorer/
	private function setFilters(){
	
		// post content hooks
		add_filter('the_content',array($this,'filter'),1001);
		add_filter('the_excerpt',array($this,'filter'),1001);

		// comment content hooks
		add_filter('comment_text',array($this,'filter'),1001);
		add_filter('comment_text_rss',array($this,'filter'),1001);
		add_filter('comment_excerpt',array($this,'filter'),1001);
		add_filter('comment_url',array($this,'filter'),1001);

		// comment author hooks
		add_filter('get_comment_author_url_link',array($this,'filter'),1001);
		add_filter('get_comment_author_link',array($this,'filter'),1001);
		add_filter('get_comment_author_url',array($this,'filter'),1001);
		
		// text widget hook
		add_filter('widget_text',array($this,'filter'),1001);
		
		// navt custom hook
		//add_filter('navt_codeblock',array($this,'filter'),1001);
		
		
		// adds our js and css to be loaded by browser
		add_action('wp_print_scripts', array($this,'scriptsAction'));
		add_action('wp_print_styles', array($this,'stylesAction'));
		add_action('wp_head', array($this,'header'),5);
        add_action('wp_footer', array($this,'printLinksCalls'),15);
		
		
		$this->links=array();
		$this->ids=array();
		$this->true_page=false;
	
	}
	
	public function header(){
		$this->true_page=true;
	}
	
	public function scriptsAction(){
		if (!is_admin()){
			wp_enqueue_script('HkMuob_jscript', $this->plugin_url.'/HkMuob.js',null,null,true);
		}
	}
	public function stylesAction(){
		wp_register_style('HkMuob_styling', $this->plugin_url.'/HkMuob.css');
		wp_enqueue_style( 'HkMuob_styling');
	}
    
    public function printLinksCalls(){
        
		$script = "\n\n<!-- Emails and URLs obfuscated by \n Hikari Email & URL Obfuscator - http://Hikari.ws/email-url-obfuscator -->" .
		"\n".'<script type="text/javascript">'."\n /* <![CDATA[ */ ";
			foreach($this->links as $link){
				$script .= "\n" . $link;
			}
		$script .= "\n /* ]]> */ \n</script>\n\n";
		
		print $script;
    }
	

/*
	$matches mapping:
	
	0: full link string
	1: before href
	2: href URL
	3: after href
	4: content

*/
	
	// this is our main filter, it receives a string as parameter and searches for <a> link patterns on it,
	//returning the string with those links obfuscated and with JavaScript calls to convert them back to links in JavaScript enabled browsers
	public function filter($content){
	
		$this->specialParameters=array(
			has_special_parameter => false,
			force_obfuscate => false,
			no_obfuscate => false,
			obfuscate_content => false,
			content_only => false,
			url_only => false,
			invisible_for_noscript => false
		
		);
		
	
		// these chars break JavaScript parameters, let's just replace them for simple space
		//$nl   = array("\n", "\r", "\t");
		//$content = str_replace($nl,' ',$content);
	
		/*
			i = Ignore case in matching
			s = . to match everything including new line (otherwise it matches everything except new line)
			see http://www.wellho.net/regex/perl.html
		*/
		$pattern = '/<a ([^>]*?)href=[\"\'](.*?)[\"\'](.*?)>(.*?)<\/a>/is';
		$content = preg_replace_callback($pattern,array($this,'parser'),$content);
		
		
		unset($this->specialParameters);
		return $content;
	
	}
	
	// our parser is called once for each found link, converts it to an obfuscated link that will only be understanded by humans
	// and also prepares the JavaScript call with required info to convert this obfuscated link back to original link
	private function parser($matches){
	
		// first, let's convert this $matches array to more meaningful named variables 
		$attibutes = $matches[1].' '.$matches[3];
		$original_URL = $matches[2];
		$original_content = $matches[4];
		
		
		// reseting, so that later we know if parseSpecialParameters() was called and they were set
		$this->specialParameters['has_special_parameter'] = false;
		$this->jsObfuscation	= 'not_set';
		$this->cssObfuscation	= 'not_set';
		
		
		// now let's search for special parameters that may had been passed specially to our plugin
		// if any parameter is passed, it will be parsed to be used later
		$specialParametersPattern = '/(.*?)hkmuob=[\"\'](.*?)[\"\'](.*?)/i';
		$attibutes = preg_replace_callback($specialParametersPattern,
				array($this,'parseSpecialParameters'),$attibutes);
		
		// if this link doesn't have any special parameter, it won't trigger parseSpecialParameters()
		//so we must set them back to their default, or those parameters will remain from last link that had any
		if(!$this->specialParameters['has_special_parameter']){
			$this->specialParameters['force_obfuscate'] 	= false;
			$this->specialParameters['no_obfuscate'] 		= false;
			$this->specialParameters['obfuscate_content'] 	= false;
			$this->specialParameters['content_only'] 		= false;
			$this->specialParameters['url_only'] 			= false;
			$this->specialParameters['invisible_for_noscript'] 	= false;
		}
		
		
		if($this->jsObfuscation	== 'not_set'){
			$this->jsObfuscation='cc8b';
			if(rand(0,1)==1)
				$this->jsObfuscation='rot13';
		}
		
		// reversion is currently broken, &---; identifiers and '=' chars invalidate XHTML when reverted
		// until I find a solution for it, we'll have to stick with display:none obfuscation
		/*
		if($this->cssObfuscation	== 'not_set'){
			$this->cssObfuscation='display_none';
			if(rand(0,1)==1)
				$this->cssObfuscation='reversion';
		}
		*/
		$this->cssObfuscation='display_none';
		
		
		// now we test if this specific link should be excluded and left alone, there are  bunch of cases that make it happen
		if($this->isExclude($matches, $attibutes, $original_URL, $original_content))
				return '<a href="' . $original_URL . '" ' . $attibutes . ' >' . $original_content . '</a>';
		// we must rebuild the link because original raw one could have HkMuob special parameters,
		//and they must be removed because their attribute isn't HTML valid
		
		

		/*
			ok, all preparations are set and now we can start our obfuscation work
			basically what we will do is 4 things: obfuscate the link in a way CSS can de-obfuscate, and it will be used on browsers that have JavaScript disabled
				this will be done in 2 possible ways: reverting characters ordering or adding strings that CSS will make not be rendered
			and obfuscate it again using ROT13 or cc8b encoding, which will be later (in user's browser) decoded back by JavaScript
		*/
		
		
		/*
			first we will encode-obfuscate our stuff, whether it will be ROT13 or cc8b was already defined by parameter or randomly
			when we verified any special parameters, so we are already ready here
		*/
		//$this->jsObfuscation="cc8b";	// hardcode for testing purpose
		
		$js_content=$original_content;
		
		if($this->jsObfuscation=='rot13'){
		
			// $key=0 sets to JavaScript we are using ROT13, since only cc8b used a variable key (ROT13's key is always 13 :P )
			$key=0;
			
			// PHP has a buil't in ROT13 encoding function, so we can just call it, yay!
			$js_url = str_rot13($original_URL);
			$js_attibutes = str_rot13($attibutes);
			
			// if we were required to obfuscate content too, now is the time to finish the job encoding it with ROT13
			if($this->specialParameters['obfuscate_content']){
				$js_content=str_rot13($original_content);
			}
			
		// now, if cc8b was the chosed encoding algorythm...
		}else{
		
			//get a random number from 10 to 254, which defines to JavaScript we are using cc8b
			$key = rand(10, 254);
		
			// get our URL encoded by our custom function (yeah I know the right name is method!)
			$js_url = $this->cc8b_encode($key,$original_URL);
			$js_attibutes = $this->cc8b_encode($key,$attibutes);
			
			// and if content obfuscation was required...
			if($this->specialParameters['obfuscate_content']){
				$js_content = $this->cc8b_encode($key,$original_content);
			}
			
		}

		
		
		
		/*
			now we'll obfuscate our link in a way that CSS can de-obfuscate it
			the disadivantage of this approach is that  we can't have a clickable link, it will be just visible text in the screen,
				and sometimes this text can't be copied
		*/
		
		$css_content=$original_content;
		
		if($this->cssObfuscation=='reversion'){
			
			$css_url = '<span class="HkMuob_revert">'.strrev(
							str_replace('mailto:','',$original_URL)		// reverted url
							).'</span>';
			
			if($this->specialParameters['obfuscate_content']){
				$css_content = '<span class="HkMuob_revert">'.
							strrev($original_content).'</span>';
							
				$css_both = '<span class="HkMuob_revert">'.
							strrev($original_content.' )'.str_replace('mailto:','',$original_URL).'(')
					.'</span>';
			}else{
				$css_both = $original_content . ' ('. $css_url . ')';
			}
		
		}else{
		
			// first we remove any 'mailto:', it's not needed when we don't have a <a> link
			$css_url = str_replace('mailto:','',$original_URL);
			
			// now we add text around any '@' and '.'
			// I could use rand(1,10000) to generate a random text, but I don't think worthy the required processing time to do it
			$css_url = str_replace('.','<span class="HkMuob_display"> NULL</span>.',$css_url);
			$css_url = str_replace('@',
				'<span class="HkMuob_display"> null</span>@<span class="HkMuob_display">null </span>'
				,$css_url);
			
			if($this->specialParameters['obfuscate_content']){
				$css_content = str_replace('.',
					'<span class="HkMuob_display"> NULL</span>.',$css_content);
				$css_content = str_replace('@',
					'<span class="HkMuob_display"> null</span>@<span class="HkMuob_display">null </span>'
					,$css_content);
			}
			
			$css_both = $css_content . ' ('. $css_url . ')';
		
		}
		
		
		
		
		
		
		
		
		
		// all obfuscation set, now let's start building our non-JavaScript code!
		
		
		// first, the code that will be seen by our non-JavaScript visitors
		$noscript_text='&nbsp;';
		
		// if it is meant to be invisible to them,  $noscript_text will remain empty
		if(!$this->specialParameters['invisible_for_noscript']){
		
			if($this->specialParameters['url_only'] && !$this->specialParameters['content_only']){
				$noscript_text = $css_url;
			}elseif($this->specialParameters['content_only'] && !$this->specialParameters['url_only']){
					$noscript_text = $css_content;
			}else{
				$noscript_text = $css_both;
			}
		
		}
		
		
		// this ID will be used by JavaScript to find the right place to include our decoded link,
		// so each link should have a unique ID in the whole document (HTML also forbids 2 elements with equal ID
		$id = 'hkmuob-id-' . $this->generateId();
		
		// here is the span that will hold our revert-obfuscated string, and also that will receive our JavaScript-decoded link
		// if by a special parameter we were required to totally hide the link from non-JavaScript user-agents, we will only add a blank span to be used by JavaScript
		// otherwise we'll do our normal job
		$noscript = '<span id="'.$id.'" class="hkmuob_noscript">' . $noscript_text . '</span>';
	
	
	
	
		// these chars break JavaScript parameters, let's just replace them for simple space
		$nl   = array("\n", "\r", "\t");
		$js_url = str_replace($nl,' ',$js_url);
		$js_attibutes = str_replace($nl,' ',$js_attibutes);
		$js_content = str_replace($nl,' ',$js_content);
		
	
		// noscript obfuscation done,
		//now let's work on our JavaScript call that will later (in user's browser) finish our magic bringing original link back to life!
		$script = 'HkMuobJS.write("' . 
				$id . '",' .				//id
				$key . ",'" .				//key
				str_replace("'","\'",$js_url) . "','".			//addr
				str_replace("'","\'",$js_attibutes) . "','".	//attributes
				str_replace("'","\'",$js_content)."'," .		//content
				$this->bTOs($this->specialParameters['obfuscate_content']).
				');';
		
		
		$this->links[] = $script;
		
		// everything done, weeee!!
		// let's return both our noscript, revert-obfuscated string, together with our JavaScript call
		// this returned string will be put in place of original link
		return $noscript;
	
	}



	// verifies each and every situation where a link should be excluded from obfuscation and left alone
	private function isExclude($matches, $attibutes, $original_URL, $original_content){
	
		if($this->specialParameters['force_obfuscate']) return false;
		if($this->specialParameters['no_obfuscate']) return true;
		if(empty($matches)) 	return true;
		if(!is_array($matches))	return true;
		if(!$this->true_page)	return true;
		
	
		// if this URL points to an internal link
		if( $this->hasNeedle($original_URL, get_bloginfo("url"))  )	return true;
		if( $this->hasNeedle($original_URL, get_bloginfo("home")) )	return true;
		
		// if this URL doesn't have an '://' (from 'http://' or ''https://' or 'ftp://'), then it is a relative link to a internal resource
		// furst we must test if it is not a mail lint (mailto:)
		if( !$this->hasNeedle($original_URL, "mailto:") && !$this->hasNeedle($original_URL, '://') )
				return true;

		
		

	
	
		// if it passes from all tests, then it should not be excluded
		return false;
	}
	
	
	// our custom cc8b encoding code
	//Cc8b originally written by Debugged Interactive Designs www.debuggeddesigns.com
	//adapted by Hikari at http://Hikari.WS
	public function cc8b_encode($key,$plain_text){
		$cipher_text = '';
		//turn each character in the string into its hexidecimal ascii value
		$ascii_plain_text = (string) bin2hex($plain_text);
		//get each acii octet 
		for ($i = 0; $i <= strlen($ascii_plain_text) - 2; $i += 2) {
			//get next octet
			$temp = substr($ascii_plain_text, $i, 2);
			//turn it to decimal
			$temp = hexdec($temp);

			//add key value to temp
			$temp += $key;
			//mod temp by 255 so its value is at most FF
			$temp = $temp % 255;
			
			//concatinate the hex value to ecrypted email
			//check to make sure we have 2 digits
			//if its 1 digit place a leading 0 infront of it (0 -> F)
			if (strlen((string) dechex($temp)) < 2) {
				$cipher_text .= '0' . (string) dechex($temp);
			}
			//else just concate the 2 digit string
			else {
				$cipher_text .= (string) dechex($temp);
			}
		}
		
		return $cipher_text;		
	}
	
	
/*
	$matches mapping:
	
	0: full attributes string
	1: before special attribute
	2: special parameters
	3: after special attribute

*/
	
	// for each link, there may be a custom attribute with special parameters to be used by Hikari Email & URL Obfuscator plugin
	// here we parte and set each of those parameters, and return a string with all original link attributes this non-standard attribute stripped out
	private function parseSpecialParameters($matches){
	
		$sp=$matches[2];
		
		$this->specialParameters['has_special_parameter'] = true;
	
		$this->specialParameters['force_obfuscate'] 		= $this->hasNeedle($sp,'force_obfuscate');
		$this->specialParameters['no_obfuscate'] 		= $this->hasNeedle($sp,'no_obfuscate');
		$this->specialParameters['obfuscate_content'] 	= $this->hasNeedle($sp,'obfuscate_content');
		$this->specialParameters['content_only'] 		= $this->hasNeedle($sp,'content_only');
		$this->specialParameters['url_only'] 			= $this->hasNeedle($sp,'url_only');
		$this->specialParameters['invisible_for_noscript'] 	= $this->hasNeedle($sp,'invisible_for_noscript');
	
	
		if($this->hasNeedle($sp,'cc8b')){
			$this->jsObfuscation='cc8b';
		}elseif($this->hasNeedle($sp,'rot13')){
			$this->jsObfuscation='rot13';
		}
		
		if($this->hasNeedle($sp,'display_none')){
			$this->cssObfuscation='display_none';
		}elseif($this->hasNeedle($sp,'reversion')){
			$this->cssObfuscation='reversion';
		}
	
	
		$newAttributes = $matches[1].' '.$matches[3];
	
		return trim($newAttributes);
	}

	// imagine what would happen if rand() renerated the same ID twice in the same filter loop...
	// no, we can't let that happen! can we? :D
	private function generateId(){
		do{
			$new_id = rand(1,10000);
		}while(in_array($new_id,$this->ids));
	
		$this->ids[] = $new_id;
		return $new_id;
	}

	// just a more abstracted replacement for strpos
	// returns true if $needle was found, false otherwhise
	public function hasNeedle($haystack,$needle){
		$found = strpos($haystack,$needle) !== false;
		return $found;
	}
	
	// bool to string, to be passed as parameter to JavaScript
	function bTOs($bool){
		if($bool)	return "true";
		else		return "false";
	}

}

global $hkMuob;
$hkMuob = new HkMuob();
